@@ -33,6 +33,7 @@ from .pairing import AbstractPairing
 if IP_TRANSPORT_SUPPORTED:
     from .ip import IpDiscovery, IpPairing
     from .ip.zeroconf import async_discover_homekit_devices
+    from aiohomekit.zeroconf import async_find_data_for_device_id
 
 
 class Controller(object):
@@ -40,13 +41,14 @@ class Controller(object):
     This class represents a HomeKit controller (normally your iPhone or iPad).
     """
 
-    def __init__(self, ble_adapter: str = "hci0") -> None:
+    def __init__(self, ble_adapter: str = "hci0", zeroconf_instance=None) -> None:
         """
         Initialize an empty controller. Use 'load_data()' to load the pairing data.
 
         :param ble_adapter: the bluetooth adapter to be used (defaults to hci0)
         """
         self.pairings = {}
+        self._zeroconf_instance = zeroconf_instance
         self.ble_adapter = ble_adapter
         self.logger = logging.getLogger(__name__)
 
@@ -78,18 +80,23 @@ class Controller(object):
         """
         if not IP_TRANSPORT_SUPPORTED:
             raise TransportNotSupportedError("IP")
-        devices = await async_discover_homekit_devices(max_seconds)
+        devices = await async_discover_homekit_devices(
+            max_seconds, zeroconf_instance=self._zeroconf_instance
+        )
         tmp = []
         for device in devices:
             tmp.append(IpDiscovery(self, device))
         return tmp
 
     async def find_ip_by_device_id(self, device_id, max_seconds=10):
-        results = await self.discover_ip(max_seconds=max_seconds)
-        for result in results:
-            if result.device_id == device_id:
-                return result
-        raise AccessoryNotFoundError("No matching accessory found")
+        if not IP_TRANSPORT_SUPPORTED:
+            raise TransportNotSupportedError("IP")
+        device = await async_find_data_for_device_id(
+            device_id=device_id,
+            max_seconds=max_seconds,
+            zeroconf_instance=self._zeroconf_instance,
+        )
+        return IpDiscovery(self, device)
 
     @staticmethod
     async def discover_ble(max_seconds=10, adapter="hci0"):
@@ -131,7 +138,7 @@ class Controller(object):
         if pairing_data["Connection"] == "IP":
             if not IP_TRANSPORT_SUPPORTED:
                 raise TransportNotSupportedError("IP")
-            pairing = self.pairings[alias] = IpPairing(pairing_data)
+            pairing = self.pairings[alias] = IpPairing(self, pairing_data)
             return pairing
 
         if pairing_data["Connection"] == "BLE":
