@@ -15,16 +15,19 @@
 #
 
 from binascii import hexlify
-import hashlib
 import logging
 from typing import Any, Dict, Generator, List, Tuple, Union
 
+from cryptography import exceptions as cryptography_exceptions
 from cryptography.hazmat.primitives import serialization
-from cryptography.hazmat.primitives.asymmetric import x25519
-import ed25519
-import hkdf
+from cryptography.hazmat.primitives.asymmetric import ed25519, x25519
 
-from aiohomekit.crypto import SrpClient, chacha20_aead_decrypt, chacha20_aead_encrypt
+from aiohomekit.crypto import (
+    SrpClient,
+    chacha20_aead_decrypt,
+    chacha20_aead_encrypt,
+    hkdf_derive,
+)
 from aiohomekit.exceptions import (
     AuthenticationError,
     BackoffError,
@@ -207,33 +210,35 @@ def perform_pair_setup_part2(
     # M5 Request generation (page 44)
     session_key = srp_client.get_session_key()
 
-    ios_device_ltsk, ios_device_ltpk = ed25519.create_keypair()
+    ios_device_ltsk = ed25519.Ed25519PrivateKey.generate()
+    ios_device_ltpk = ios_device_ltsk.public_key()
+    ios_device_public_bytes = ios_device_ltpk.public_bytes(
+        encoding=serialization.Encoding.Raw, format=serialization.PublicFormat.Raw
+    )
 
     # reversed:
     #   Pair-Setup-Encrypt-Salt instead of Pair-Setup-Controller-Sign-Salt
     #   Pair-Setup-Encrypt-Info instead of Pair-Setup-Controller-Sign-Info
-    hkdf_inst = hkdf.Hkdf(
-        "Pair-Setup-Controller-Sign-Salt".encode(),
+    ios_device_x = hkdf_derive(
         SrpClient.to_byte_array(session_key),
-        hash=hashlib.sha512,
+        "Pair-Setup-Controller-Sign-Salt",
+        "Pair-Setup-Controller-Sign-Info",
     )
-    ios_device_x = hkdf_inst.expand("Pair-Setup-Controller-Sign-Info".encode(), 32)
 
-    hkdf_inst = hkdf.Hkdf(
-        "Pair-Setup-Encrypt-Salt".encode(),
+    session_key = hkdf_derive(
         SrpClient.to_byte_array(session_key),
-        hash=hashlib.sha512,
+        "Pair-Setup-Encrypt-Salt",
+        "Pair-Setup-Encrypt-Info",
     )
-    session_key = hkdf_inst.expand("Pair-Setup-Encrypt-Info".encode(), 32)
 
     ios_device_pairing_id = ios_pairing_id.encode()
-    ios_device_info = ios_device_x + ios_device_pairing_id + ios_device_ltpk.to_bytes()
+    ios_device_info = ios_device_x + ios_device_pairing_id + ios_device_public_bytes
 
     ios_device_signature = ios_device_ltsk.sign(ios_device_info)
 
     sub_tlv = [
         (TLV.kTLVType_Identifier, ios_device_pairing_id),
-        (TLV.kTLVType_PublicKey, ios_device_ltpk.to_bytes()),
+        (TLV.kTLVType_PublicKey, ios_device_public_bytes),
         (TLV.kTLVType_Signature, ios_device_signature),
     ]
     sub_tlv_b = TLV.encode_list(sub_tlv)
@@ -244,10 +249,11 @@ def perform_pair_setup_part2(
     encrypted_data_with_auth_tag = chacha20_aead_encrypt(
         bytes(), session_key, "PS-Msg05".encode(), bytes([0, 0, 0, 0]), sub_tlv_b
     )
-    tmp = bytearray(encrypted_data_with_auth_tag[0])
-    tmp += encrypted_data_with_auth_tag[1]
 
-    response_tlv = [(TLV.kTLVType_State, TLV.M5), (TLV.kTLVType_EncryptedData, tmp)]
+    response_tlv = [
+        (TLV.kTLVType_State, TLV.M5),
+        (TLV.kTLVType_EncryptedData, encrypted_data_with_auth_tag),
+    ]
 
     step6_expectations = [
         TLV.kTLVType_State,
@@ -300,27 +306,32 @@ def perform_pair_setup_part2(
     ), "perform_pair_setup: No public key"
     accessory_ltpk = response_tlv[1][1]
 
-    hkdf_inst = hkdf.Hkdf(
-        "Pair-Setup-Accessory-Sign-Salt".encode(),
+    accessory_x = hkdf_derive(
         SrpClient.to_byte_array(srp_client.get_session_key()),
-        hash=hashlib.sha512,
+        "Pair-Setup-Accessory-Sign-Salt",
+        "Pair-Setup-Accessory-Sign-Info",
     )
-    accessory_x = hkdf_inst.expand("Pair-Setup-Accessory-Sign-Info".encode(), 32)
 
     accessory_info = accessory_x + accessory_pairing_id + accessory_ltpk
 
-    e25519s = ed25519.VerifyingKey(bytes(response_tlv[1][1]))
+    e25519s = ed25519.Ed25519PublicKey.from_public_bytes(bytes(response_tlv[1][1]))
     try:
         e25519s.verify(bytes(accessory_sig), bytes(accessory_info))
-    except AssertionError:
+    except cryptography_exceptions.InvalidSignature:
         raise InvalidSignatureError("step #7")
 
+    ios_device_ltsk_private_bytes = ios_device_ltsk.private_bytes(
+        encoding=serialization.Encoding.Raw,
+        format=serialization.PrivateFormat.Raw,
+        encryption_algorithm=serialization.NoEncryption(),
+    )
+
     return {
         "AccessoryPairingID": response_tlv[0][1].decode(),
         "AccessoryLTPK": hexlify(response_tlv[1][1]).decode(),
         "iOSPairingId": ios_pairing_id,
-        "iOSDeviceLTSK": ios_device_ltsk.to_ascii(encoding="hex").decode()[:64],
-        "iOSDeviceLTPK": hexlify(ios_device_ltpk.to_bytes()).decode(),
+        "iOSDeviceLTSK": ios_device_ltsk_private_bytes.hex(),
+        "iOSDeviceLTPK": ios_device_public_bytes.hex(),
     }
 
 
@@ -383,10 +394,9 @@ def get_session_keys(
     shared_secret = ios_key.exchange(accessorys_session_pub_key)
 
     # 2) derive session key
-    hkdf_inst = hkdf.Hkdf(
-        "Pair-Verify-Encrypt-Salt".encode(), shared_secret, hash=hashlib.sha512
+    session_key = hkdf_derive(
+        shared_secret, "Pair-Verify-Encrypt-Salt", "Pair-Verify-Encrypt-Info"
     )
-    session_key = hkdf_inst.expand("Pair-Verify-Encrypt-Info".encode(), 32)
 
     # 3) verify auth tag on encrypted data and 4) decrypt
     encrypted = response_tlv[2][1]
@@ -406,7 +416,9 @@ def get_session_keys(
     if pairing_data["AccessoryPairingID"] != accessory_name:
         raise IncorrectPairingIdError("step 3")
 
-    accessory_ltpk = ed25519.VerifyingKey(bytes.fromhex(pairing_data["AccessoryLTPK"]))
+    accessory_ltpk = ed25519.Ed25519PublicKey.from_public_bytes(
+        bytes.fromhex(pairing_data["AccessoryLTPK"])
+    )
 
     # 6) verify accessory's signature
     accessory_sig = d1[1][1]
@@ -416,7 +428,7 @@ def get_session_keys(
     )
     try:
         accessory_ltpk.verify(bytes(accessory_sig), bytes(accessory_info))
-    except ed25519.BadSignatureError:
+    except cryptography_exceptions.InvalidSignature:
         raise InvalidSignatureError("step 3")
 
     # 7) create iOSDeviceInfo
@@ -428,10 +440,14 @@ def get_session_keys(
 
     # 8) sign iOSDeviceInfo with long term secret key
     ios_device_ltsk_h = pairing_data["iOSDeviceLTSK"]
-    ios_device_ltpk_h = pairing_data["iOSDeviceLTPK"]
-    ios_device_ltsk = ed25519.SigningKey(
-        bytes.fromhex(ios_device_ltsk_h) + bytes.fromhex(ios_device_ltpk_h)
+    # ios_device_ltpk_h = pairing_data["iOSDeviceLTPK"]
+
+    ios_device_ltsk = ed25519.Ed25519PrivateKey.from_private_bytes(
+        bytes.fromhex(ios_device_ltsk_h)
     )
+    # ios_device_ltsk = ed25519.SigningKey(
+    #    bytes.fromhex(ios_device_ltsk_h) + bytes.fromhex(ios_device_ltpk_h)
+    # )
     ios_device_signature = ios_device_ltsk.sign(ios_device_info)
 
     # 9) construct sub tlv
@@ -446,11 +462,12 @@ def get_session_keys(
     encrypted_data_with_auth_tag = chacha20_aead_encrypt(
         bytes(), session_key, "PV-Msg03".encode(), bytes([0, 0, 0, 0]), sub_tlv
     )
-    tmp = bytearray(encrypted_data_with_auth_tag[0])
-    tmp += encrypted_data_with_auth_tag[1]
 
     # 11) create tlv
-    request_tlv = [(TLV.kTLVType_State, TLV.M3), (TLV.kTLVType_EncryptedData, tmp)]
+    request_tlv = [
+        (TLV.kTLVType_State, TLV.M3),
+        (TLV.kTLVType_EncryptedData, encrypted_data_with_auth_tag),
+    ]
 
     step3_expectations = [TLV.kTLVType_State, TLV.kTLVType_Error]
     response_tlv = yield (request_tlv, step3_expectations)
@@ -466,14 +483,13 @@ def get_session_keys(
         error_handler(response_tlv[1][1], "verification")
 
     # calculate session keys
-    hkdf_inst = hkdf.Hkdf("Control-Salt".encode(), shared_secret, hash=hashlib.sha512)
-    controller_to_accessory_key = hkdf_inst.expand(
-        "Control-Write-Encryption-Key".encode(), 32
+
+    controller_to_accessory_key = hkdf_derive(
+        shared_secret, "Control-Salt", "Control-Write-Encryption-Key"
     )
 
-    hkdf_inst = hkdf.Hkdf("Control-Salt".encode(), shared_secret, hash=hashlib.sha512)
-    accessory_to_controller_key = hkdf_inst.expand(
-        "Control-Read-Encryption-Key".encode(), 32
+    accessory_to_controller_key = hkdf_derive(
+        shared_secret, "Control-Salt", "Control-Read-Encryption-Key"
     )
 
     return controller_to_accessory_key, accessory_to_controller_key
